@@ -43,22 +43,30 @@ static void xconfigAddRemovedOptionComment(char **existing_comments,
 {
     int len;
     char *str;
+    char *name, *value;
 
     if (!option || !existing_comments)
         return;
 
-    len = 32 + strlen(xconfigOptionName(option)) +
-        strlen(xconfigOptionValue(option));
+    name = xconfigOptionName(option);
+    value = xconfigOptionValue(option);
 
-    str = (char *)malloc(len);
+    if (!name) return;
 
-    if (str) {
-        snprintf(str, len, "# Removed Option \"%s\" \"%s\"",
-                xconfigOptionName(option),
-                xconfigOptionValue(option));
-        *existing_comments = xconfigAddComment(*existing_comments, str);
+    if (value) {
+        len = 32 + strlen(name) + strlen(value);
+        str = malloc(len);
+        if (!str) return;
+        snprintf(str, len, "# Removed Option \"%s\" \"%s\"", name, value);
+    } else {
+        len = 32 + strlen(name);
+        str = malloc(len);
+        if (!str) return;
+        snprintf(str, len, "# Removed Option \"%s\"", name);
     }
 
+    *existing_comments = xconfigAddComment(*existing_comments, str);
+
 } /* xconfigAddRemovedOptionComment() */
 
 
@@ -68,23 +76,53 @@ static void xconfigAddRemovedOptionComment(char **existing_comments,
  * list and (if specified) adds a comment to an existing comments string
  *
  */
-static void xconfigRemoveNamedOption(XConfigOptionPtr *head, char *name,
-                                     char **comments)
+void xconfigRemoveNamedOption(XConfigOptionPtr *pHead, const char *name,
+                              char **comments)
 {
     XConfigOptionPtr option;
 
-    option = xconfigFindOption(*head, name);
+    option = xconfigFindOption(*pHead, name);
     if (option) {
         if (comments) {
             xconfigAddRemovedOptionComment(comments, option);
         }
-        *head = xconfigRemoveOption(*head, option);
+        xconfigRemoveOption(pHead, option);
     }
 
 } /* xconfigRemoveNamedOption() */
 
 
 
+/*
+ * xconfigOptionValuesDiffer() - return '1' if the option values for
+ * option0 and option1 are different; return '0' if the option values
+ * are the same.
+ */
+
+static int xconfigOptionValuesDiffer(XConfigOptionPtr option0,
+                                     XConfigOptionPtr option1)
+{
+    char *value0, *value1;
+
+    value0 = value1 = NULL;
+
+    if (!option0 && !option1) return 0;
+    if (!option0 &&  option1) return 1;
+    if ( option0 && !option1) return 1;
+
+    value0 = xconfigOptionValue(option0);
+    value1 = xconfigOptionValue(option1);
+
+    if (!value0 && !value1) return 0;
+    if (!value0 &&  value1) return 1;
+    if ( value0 && !value1) return 1;
+
+    return (strcmp(value0, value1) != 0);
+
+} /* xconfigOptionValuesDiffer() */
+
+
+
 /*
  * xconfigMergeOption() - Merge option "name" from option source
  * list "srcHead" to option destination list "dstHead".
@@ -107,20 +145,35 @@ static void xconfigMergeOption(XConfigOptionPtr *dstHead,
 {
     XConfigOptionPtr srcOption = xconfigFindOption(*srcHead, name);
     XConfigOptionPtr dstOption = xconfigFindOption(*dstHead, name);
-    
-    if (!srcOption) {
-        if (dstOption) {
-            *dstHead = xconfigRemoveOption(*dstHead, dstOption);
-        }
-    } else {
-        if (!dstOption || strcmp(xconfigOptionValue(srcOption),
-                                 xconfigOptionValue(dstOption))) {
 
-            if (dstOption && comments) {
+    char *srcValue = NULL;
+
+    if (srcOption) srcValue = xconfigOptionValue(srcOption);
+
+    if (!srcOption && dstOption) {
+
+        /* option does not exist in src, but exists in dst: remove from dst */
+        xconfigRemoveOption(dstHead, dstOption);
+
+    } else if (srcOption && !dstOption) {
+
+        /* option exists in src but not in dst: add to dst */
+        xconfigAddNewOption(dstHead, name, srcValue);
+
+    } else if (srcOption && dstOption) {
+
+        /*
+         * option exists in src and in dst; if the option values are
+         * different, replace the dst's option value with src's option
+         * value; note that xconfigAddNewOption() will remove the old
+         * option first, if necessary
+         */
+
+        if (xconfigOptionValuesDiffer(srcOption, dstOption)) {
+            if (comments) {
                 xconfigAddRemovedOptionComment(comments, dstOption);
             }
-            *dstHead = xconfigAddNewOption
-                (*dstHead, name, xconfigOptionValue(srcOption));
+            xconfigAddNewOption(dstHead, name, srcValue);
         }
     }
 
@@ -206,8 +259,7 @@ static void xconfigMergeMonitors(XConfigMonitorPtr dstMonitor,
     /* XXX Remove the destination monitor's "UseModes" references to
      *     avoid having the wrong modelines tied to the new monitor.
      */
-    xconfigFreeModesLinkList(dstMonitor->modes_sections);
-    dstMonitor->modes_sections = NULL;
+    xconfigFreeModesLinkList(&dstMonitor->modes_sections);
 
 } /* xconfigMergeMonitors() */
 
@@ -242,9 +294,8 @@ static int xconfigMergeAllMonitors(XConfigPtr dstConfig, XConfigPtr srcConfig)
 
             dstMonitor->identifier = xconfigStrdup(srcMonitor->identifier);
 
-            dstConfig->monitors = (XConfigMonitorPtr)
-                xconfigAddListItem((GenericListPtr)dstConfig->monitors,
-                                   (GenericListPtr)dstMonitor);
+            xconfigAddListItem((GenericListPtr *)(&dstConfig->monitors),
+                               (GenericListPtr)dstMonitor);
         }
 
         /* Do the merge */
@@ -333,9 +384,8 @@ static int xconfigMergeAllDevices(XConfigPtr dstConfig, XConfigPtr srcConfig)
 
             dstDevice->identifier = xconfigStrdup(srcDevice->identifier);
 
-            dstConfig->devices = (XConfigDevicePtr)
-                xconfigAddListItem((GenericListPtr)dstConfig->devices,
-                                   (GenericListPtr)dstDevice);
+            xconfigAddListItem((GenericListPtr *)(&dstConfig->devices),
+                               (GenericListPtr)dstDevice);
         }
 
         /* Do the merge */
@@ -387,21 +437,19 @@ static int xconfigMergeDriverOptions(XConfigScreenPtr dstScreen,
             XConfigOptionPtr old =
                 xconfigFindOption(dstScreen->options, name);
 
-            if (!old || !strcmp(xconfigOptionValue(option),
-                                xconfigOptionValue(old))) {
-                xconfigRemoveNamedOption(&(dstScreen->options), name,
-                                         NULL);
-            } else {
+            if (old && xconfigOptionValuesDiffer(option, old)) {
                 xconfigRemoveNamedOption(&(dstScreen->options), name,
                                          &(dstScreen->comment));
+            } else {
+                xconfigRemoveNamedOption(&(dstScreen->options), name,
+                                         NULL);
             }
         }
 
         /* Add the option to the screen->options list */
 
-        dstScreen->options =
-            xconfigAddNewOption(dstScreen->options,
-                                name, xconfigOptionValue(option));
+        xconfigAddNewOption(&dstScreen->options,
+                            name, xconfigOptionValue(option));
         
         option = option->next;
     }
@@ -426,7 +474,7 @@ static int xconfigMergeDisplays(XConfigScreenPtr dstScreen,
 
     /* Free all the displays in the destination screen */
 
-    xconfigFreeDisplayList(dstScreen->displays);
+    xconfigFreeDisplayList(&dstScreen->displays);
 
     /* Copy all te displays */
     
@@ -461,11 +509,12 @@ static int xconfigMergeDisplays(XConfigScreenPtr dstScreen,
 
         lastDstMode = NULL;
         srcMode = srcDisplay->modes;
+        dstMode = NULL;
         while (srcMode) {
 
             /* Copy the mode */
-
-            dstMode = xconfigAddMode(NULL, srcMode->mode_name);
+            
+            xconfigAddMode(&dstMode, srcMode->mode_name);
 
             /* Add mode at the end of the list */
 
@@ -478,6 +527,9 @@ static int xconfigMergeDisplays(XConfigScreenPtr dstScreen,
 
             srcMode = srcMode->next;
         }
+
+        xconfigAddListItem((GenericListPtr *)(&dstScreen->displays),
+                           (GenericListPtr)dstDisplay);
     }
 
     return 1;
@@ -562,9 +614,8 @@ static int xconfigMergeAllScreens(XConfigPtr dstConfig, XConfigPtr srcConfig)
 
             dstScreen->identifier = xconfigStrdup(srcScreen->identifier);
 
-            dstConfig->screens = (XConfigScreenPtr)
-                xconfigAddListItem((GenericListPtr)dstConfig->screens,
-                                   (GenericListPtr)dstScreen);
+            xconfigAddListItem((GenericListPtr *)(&dstConfig->screens),
+                               (GenericListPtr)dstScreen);
         }
 
         /* Do the merge */
@@ -593,7 +644,7 @@ static int xconfigMergeLayout(XConfigPtr dstConfig, XConfigPtr srcConfig)
 
     /* Clear the destination's adjacency list */
 
-    xconfigFreeAdjacencyList(dstLayout->adjacencies);
+    xconfigFreeAdjacencyList(&dstLayout->adjacencies);
     
     /* Copy adjacencies over */
     
